
# q-expansion of newform 432.8.a.i, downloaded from the LMFDB on 13 March 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 432
weight = 8
poly_data = [-322, -1, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0], 1], [[-9, 18], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0], [0, 0], [-162, -1], [490, 3], [405, 4], [-3940, -24], [10368, 40], [-5228, -30], [-30294, -40], [-16200, -622], [72994, 405], [-5230, -192], [542700, 410], [303220, 1782], [-747954, 840], [1039878, -1375], [-1537380, 2000], [300560, 2520], [1214620, 2358], [-1600560, 4472], [548765, -6732], [756580, 1860], [-1880091, -4740], [-5392980, -15746], [1494455, 18552], [-11995290, -13427], [3840820, -30384], [-3885651, -65480], [9197066, -29160], [-13692564, 26730], [-22887650, 34101], [11664405, 35440], [-30335796, -24010], [34266544, -2940], [-5876550, 100435], [1825546, -44055], [-46958260, 19440], [-69757280, 17364], [-29463750, 215280], [33481998, 193755], [-82840725, 105160], [55678040, -222600], [-102567060, 157472], [-42668215, 195948], [-12332574, -107675], [-131172434, -137955], [4259212, -595470], [-96557300, -590004], [-47588148, 405480], [-79631482, -259200], [-71801316, 284570], [-91569690, -167184], [-81232618, -299280], [124237800, 727864], [-212348952, 287700], [-145552302, -1221880], [-184307400, 1141178], [56893726, -239265], [183127400, 1349136], [-201472920, 859448], [-36270500, 247374], [-458610336, -430730], [276938800, -485352], [-110551230, -919184], [-103521775, -244380], [44415702, 1285525], [4639252, 1802310], [705666170, -560352], [-712018917, -2189220], [227534630, 1698000], [124669044, -3438010], [-527623470, -711264], [-603429350, -1260207], [1246501640, 1337952], [204700960, -3657300], [259686000, 3675760], [524553570, -1223885], [-1031254180, -255312], [1508790240, 2504324], [25586975, -7772880], [-211920300, -3569304], [-475111840, 925080], [846651690, -1300024], [-791476675, -1319328], [-110335562, 3932145], [1471923900, 1122600], [-109228500, -6088122], [-995534065, -1782648], [-385426350, 4559963], [240550270, -6634461], [1740634677, -681980], [1427034510, -4888824], [-938954780, 2253372], [1544663115, -8467744], [3291198484, 3851550], [1495644102, -5675280], [-375628590, 5075477], [1306792440, 3341644], [-730837520, 15116472], [393238772, -11375040], [-2777962280, -7694568], [2028896586, 9165715], [-2180885877, -9438060], [5047349760, 3649228], [4339945528, -6756540], [3368153510, -4238352], [-595911735, 14427200], [4843911132, -2466710], [-4751052570, -8115952], [-3517667425, 950940], [4014120940, -11272956], [-1435048930, -20395224], [7739649468, 2073710], [549915736, 29562300], [-5138003078, 4126395], [1042050420, -11105854], [6269531080, -14372964], [-377676432, 32348640], [-1479207906, 11837875], [-3506796585, 6665456], [90446870, -27867960], [2354331245, 27319296], [-2364443784, 15513790], [-2611566684, -3195960], [-4526348408, -10421460], [-14366431350, -3148219], [-2062872364, 16300200], [-2212694820, -12322968], [-5101462430, 20927403], [10068660410, -8996832], [-3429681464, -23036400], [-1582623036, -7363440], [7289578114, 8462145], [-8661797830, 2745360], [-8036398800, 7975056], [12682959725, -19830300], [-11935901448, 29489130], [-6319851680, -17487360], [-8670542778, 19075275], [6398463780, 497362], [13629724468, 3665250], [-22937025960, 1266322], [-13004044670, -35489199], [-12059268336, 11840240], [-11950222366, 42207960], [-7483486320, 14946192], [-13755648010, -15370344], [25119802296, 10876560], [9920390128, 20134920], [-2130289308, 48998840], [12156907250, -56331264], [-4913952480, -64457860], [21494085220, 13456482], [-14964257196, 37064560], [-4458189380, -32066814], [-3740710410, -26589072], [5269992730, -9690435], [23969461680, -27650628], [-15877124965, 2983848], [-9467829990, 56081935], [-4949965161, 79790520], [14071719816, -102463140], [21630055390, 6946695], [8679011085, -110113452], [34122514572, -26619290], [18395049294, 112402520], [2813296762, -13244805], [30843716300, 83138400], [14793731819, -98919360], [-14270644152, 26068670], [1217257875, -60374672], [-33841446322, 11054880], [31610761110, -107379176], [-21841834630, -89620608], [-25014356174, -136852455], [31070088540, 75095626], [-8816686892, 111171690], [-52140225690, -18747885], [-50579754650, 46017447], [-7215789808, 92472840], [6817600930, -90497391], [48764658825, -104976004], [-23232490690, -63419760], [33760231056, -12490740], [19928981772, -42585800], [2110711770, -4858025], [40886260550, -140646384], [2517848680, -216404088], [-74849913529, -2112180], [60435082530, 53322944], [32264471330, 132530160], [-13161066381, -36812280], [-63606262940, -62724390], [-51654889890, -51845789], [-40035293604, 45019760], [19330627320, -232314740], [-21075531181, -26319660], [26862259190, -159967104], [62340606720, -108197060], [-49894137972, -184087000], [25064281170, -161921841], [77302675534, -6508875], [-73792958920, 122634912], [11900091857, -9101700], [-19376928945, 42739468], [86127517080, 42475930], [-37949264972, 189821400], [-68112709083, -204115820], [69642712980, -79965758], [-14782413584, -245024880], [15206201765, 230919108], [-42396495120, 170398954], [45304371310, -141998373], [-33215443929, 83502680], [-40501762560, 127525944], [115077456905, 37605960], [-22114282460, 445867740], [-84069188820, 41977554], [-1156708998, 324479320], [18044787960, 83838430], [-3301902394, -192654600], [-57906838364, -229482720], [-5964169320, 331121024], [87904853620, -39723048], [-20741789151, 142028220], [150710272016, 53334840], [-20461472856, 617091600], [-48722963670, -253955720], [130242376300, -73463316], [108696798180, -162644720], [-64614100210, 140653752], [158414663896, 79211520], [-96555767684, 428870700], [-164427307560, 83274140], [-56499069740, 176370006], [-117787115844, -188529200], [69581139230, 112591920], [-106038071352, -288302250], [-55906999065, 127599060], [130316955300, 289545664], [-54270087804, 181715000], [188770110724, -232523010], [-202735896590, -164831589], [-21775251376, -452894040], [-105359916024, -56611160], [-58139303580, 101396296], [-7868095190, -281571987], [-43336464075, -88284008], [-109052168180, 106974270], [-166425734052, -124927440], [89669807420, 428698008], [-112657579920, 459436416], [-174998366946, 311408240], [9261754601, 329343120], [155641686138, 50964345], [-10932242355, 706595336], [47511314396, 629160120], [-71919140120, -153252000], [183387990870, 239230305], [106879937465, -174270528], [237139301830, -286352193], [-287599896081, 153779880], [-242371597642, -241353000], [79161210080, 358590984], [-16958539080, -697215220], [-193052988008, -48105900], [280064368800, -180953870], [-50057839620, 996329934], [140700516880, -279084852], [55598674104, -634097970], [10570830752, 206696760], [126901099720, 31878840], [-23903737795, -477200160], [-19238619926, -34657425], [-178455986455, 574246584], [-188125786250, 454546557], [-62901123075, -1083454580], [-89292074752, -314448480], [-6197731909, -61263540], [260897560740, 135555200], [196149935988, 627818320], [25470386086, -1445319855], [14815358298, -765665120], [155913325970, -90610200], [207211603660, 531072522], [432025692930, 164277648], [196588686455, -907554948], [353881441566, -113995985], [225863315860, -387877140], [10423319760, 1158865968], [108022899240, 310353582], [-162634360167, -651041720], [-132854230728, 665167780], [246665714535, -1072351656], [-186687985690, 518701968], [312846400440, -249539642], [152558340358, 602305845], [-3249663138, -690075815], [359476819425, 56473740], [-314019344300, -581113038], [174283876730, 1484378544], [9269062470, -1236837705], [-312290031638, -684168525], [-255412290879, 513148720], [-151452424028, -1741645950], [63433510715, 1080264276], [173159099460, 910802440], [117499849310, -195545880], [78239976840, 672498320], [49278779000, 45350976], [92162982924, -618106040], [414461371050, -310924563], [-100460326140, -471602070], [-155571849200, -1211948004], [-370188896976, 1086126680], [-81455205814, 42489360], [-147104608275, -979356800], [85193941860, 1056600176], [-92296857070, 1471859424], [443737493460, -21191610], [-392477081792, 1462935420], [349840102025, 12568764], [-127344445650, -1748898427], [-304141301750, 276565023], [242564793480, 987000880], [-454604218993, -176070000], [340449109972, -1397070990], [88953280120, 1954264320], [273840986520, 291090960], [514649816478, 440118755], [178208926466, 1059976080], [-11908651752, -308974610], [-214112406794, -1800291435], [-266343235128, 359777040], [644444832040, -126100800], [186482682297, -2104602740], [8063155610, 2420000280], [-310860817308, 1751692730], [94219832741, -2159869860], [170606875030, 485072955], [-504064401070, -684372696], [93740239224, 1106608520], [-332173688130, 1326918693], [146077901356, 1656867900], [-313445906874, 1881581665], [-425559930300, 554342824], [-648038687686, -1575190800], [-388202041340, 1677518262], [494575286130, 1044458336], [-565492248504, 1967784290], [-225533020232, -1590675000], [-463239098605, -904822656], [354592352160, 103684030], [301238081830, -1481901075], [422103070406, 69500880], [-891710037252, -912179470], [450349841790, -1926926040], [209958852195, 3189800576], [-474313774572, 1287706090], [-664045668270, 1020632440], [39299902160, 653512512], [-272990988720, 911142992], [31858115508, -889779840], [-36262263465, -3522092308], [-73474254260, -3994568382], [-504139883170, -211719216], [-607353596712, 865219450], [-822518372870, 1296359421], [-204024892519, 1460192520], [-1326304390275, -551990996], [652266177472, 2971286220], [-471560761962, -2918181440], [392542787250, -1965437747], [720260540746, 1252534425], [248966339300, -532250808], [-768563014680, 675262200], [-1280609890650, 518163480], [995439576635, 1331232816], [285157362060, 1966287106], [-461860983490, -1631182752], [827768308230, 1347861575], [463623282468, -3594912410], [-716867263340, 1480790244], [177403021560, -401220320], [390054510664, -3944875260], [-1201773595698, -1017242080], [837952940620, 1885297680], [40552631660, 3346741608], [-539197788860, -1636046310], [1301913272484, 220402080], [-54737936983, -2893115880], [-1044176955768, -923428970], [-383745776580, -4231521424], [-753932269580, 1642839762], [-408464828460, 2677886832], [-494701065250, 135300528], [-1300073301476, -860374860], [-185736423060, -666417774], [891542782444, -478181070], [965192866110, 3127914899], [570951503708, 3414010680], [-469815899004, -662044250], [-47930872820, -4714183476], [-349699293792, 2488318680], [344785601970, 2995532673], [-364230646958, -1482814755], [266465698250, -4172205624], [1750489431840, -490620412], [-500040486200, 847358604], [1010851276995, -2283439180], [-567984337165, 4590534060], [-1759044580074, -193083565], [109739102697, -550422860], [-1674028164272, -595558620], [-124012104235, 112621884], [-1125871634610, -3348774023], [1707385227570, -2084196368], [1612801007860, 248753220], [-553984988364, 680385890], [-36031011462, -4457596400], [327364519680, 4014982370], [-289957291270, -1637275032], [1265429984724, 1446907160], [1921260836385, -1775585964], [552802038185, 3687930324], [-531070187670, 2171786965], [542374172073, 3134032900], [549783947340, -6226450866], [371623741936, -6244021080], [632147468940, 4835166560]]
